//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MasksQModel.cpp
//! @brief     Implements class MasksQModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Mask/MasksQModel.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/Model/Mask/MasksSet.h"
#include <QColor>

// Implementation of MasksQModel is based on the Qt source code for QStringListModel

MasksQModel::MasksQModel(MasksSet* set)
    : m_masks(set)
{
}

MasksQModel::~MasksQModel() = default;

int MasksQModel::rowCount(const QModelIndex&) const
{
    return m_masks->size();
}

QVariant MasksQModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid())
        return {};
    size_t row = index.row();
    if (row >= set()->size())
        return {};
    const MaskItem* t = set()->at(row);

    switch (role) {
    case Qt::DisplayRole:
        return t->maskName();
    case Qt::BackgroundRole:
        return row == set()->currentIndex() ? QColor(Qt::cyan) : QColor(Qt::white);
    default:
        return {};
    }
}

void MasksQModel::clear()
{
    QAbstractListModel::beginResetModel();
    m_masks->clear();
    QAbstractListModel::endResetModel();

    emit set()->setChanged();
}

void MasksQModel::deleteItem()
{
    const size_t row = set()->currentIndex();
    ASSERT(row != size_t(-1));
    beginRemoveRows({}, row, row);
    set()->delete_current();
    endRemoveRows();

    emit set()->setChanged();
}

QModelIndex MasksQModel::pushItem(MaskItem* t)
{
    if (!t)
        return {};
    const size_t row = set()->size();
    beginInsertRows({}, row, row);
    set()->push_back(t);
    endInsertRows();

    emit set()->setChanged();

    return createIndex(row, 0);
}

//! Move mask to a given row
void MasksQModel::moveMask(int from_row, int to_row)
{
    emit QAbstractListModel::beginMoveRows(m_masks->rootIndex, from_row, from_row,
                                           m_masks->rootIndex, to_row);
    m_masks->swap(from_row, to_row);
    emit QAbstractListModel::endMoveRows();
}

void MasksQModel::moveUp()
{
    /*
    for (const QModelIndex& itemIndex : m_selection_model->selectedIndexes()) {
        int row = itemIndex.row();
        int new_row = row - 1;
        if (new_row >= 0 && new_row < rowCount({}))
            moveMask(row, new_row);
    }
    */
}

void MasksQModel::moveDown()
{
    /*
    for (const QModelIndex& itemIndex : m_selection_model->selectedIndexes()) {
        int row = itemIndex.row();
        int new_row = row + 1;
        if (new_row >= 0 && new_row < rowCount({}))
            // Here we lower element from 'new_row' instead of rising element from 'row'
            // The result should be the same, but rising doesn't work for an unknown reason
            moveMask(new_row, row);
    }
    */
}
